"""CLI support for nextflow pipelines."""

import click

OPTION_LOG = click.option(
    "-log",
    type=click.Path(),
    help="Set nextflow log file path",
)
OPTION_WORKDIR = click.option(
    "-work-dir",
    type=click.Path(),
    help="Directory where intermediate result files are stored",
)
OPTION_RESUME = click.option(
    "-resume",
    is_flag=True,
    default=False,
    show_default=True,
    help="Execute the script using the cached results, useful to continue \
        executions that was stopped by an error",
)
OPTION_PROFILE = click.option(
    "-profile",
    type=str,
    default="singularity,hasta,dev_prio",
    show_default=True,
    help="Choose a configuration profile",
)
OPTION_TOWER = click.option(
    "-with-tower",
    is_flag=True,
    default=False,
    show_default=True,
    help="Monitor workflow execution with Seqera Tower service",
)
OPTION_STUB = click.option(
    "-stub",
    is_flag=True,
    default=False,
    show_default=True,
    help="Execute the workflow replacing process scripts with command stubs",
)
OPTION_INPUT = click.option(
    "--input", type=click.Path(exists=True), help="Path to samplesheet containing fastq files"
)
OPTION_OUTDIR = click.option(
    "--outdir", type=click.Path(), help="Path to output folder containing analysis files"
)



"""Fixtures for the housekeeper tests."""
from pathlib import Path
from typing import Any, Dict, List

import datetime

import pytest

from cg.apps.housekeeper.hk import HousekeeperAPI
from housekeeper.store import models

from tests.mocks.hk_mock import MockHousekeeperAPI


@pytest.fixture(name="hk_config")
def fixture_hk_config(root_path: Path) -> dict:
    """Return a dictionary with housekeeper api configs for testing."""
    return {"housekeeper": {"database": "sqlite:///:memory:", "root": root_path.as_posix()}}


@pytest.fixture(scope="function", name="housekeeper_api")
def fixture_housekeeper_api(hk_config):
    """Setup Housekeeper store."""
    _api = HousekeeperAPI(hk_config)
    _api.initialise_db()
    yield _api
    _api.destroy_db()


@pytest.fixture(scope="function", name="populated_housekeeper_api")
def fixture_populated_housekeeper_api(
    housekeeper_api: MockHousekeeperAPI, hk_bundle_data: Dict[str, Any]
) -> MockHousekeeperAPI:
    """Setup Moch Housekeeper store and return API."""
    _api: MockHousekeeperAPI = housekeeper_api
    bundle_obj, version_obj = _api.add_bundle(hk_bundle_data)
    _api.add_commit(bundle_obj, version_obj)
    return _api


@pytest.fixture(name="minimal_bundle_obj")
def fixture_minimal_bundle_obj(
    timestamp: datetime.datetime, case_id: str, housekeeper_api: MockHousekeeperAPI
) -> models.Bundle:
    """Return a bundle object with minimal information (name and created_at)."""
    return housekeeper_api.new_bundle(name=case_id, created_at=timestamp)


@pytest.fixture(name="hk_tag")
def fixture_hk_tag() -> str:
    """Return a Housekeeper tag."""
    return "bed"


@pytest.fixture(name="tags")
def fixture_tags() -> List[str]:
    """Return a list of Housekeeper tags."""
    return ["bed"]


@pytest.fixture(name="not_existing_hk_tag")
def fixture_not_existing_hk_tag() -> str:
    """Return a non existing Housekeeper tag."""
    return "this_tag_should_not_exist_in_database"



import datetime as dt
import logging
from pathlib import Path

import pytest

from cg.apps.tb import TrailblazerAPI
from cg.cli.workflow.commands import clean_run_dir
from cg.constants import Pipeline
from click.testing import CliRunner

from cg.models.cg_config import CGConfig
from tests.store_helpers import StoreHelpers

EXIT_SUCCESS = 0


def test_dry_run(
    microsalt_case_clean_dry: str,
    cli_runner: CliRunner,
    clean_context_microsalt: CGConfig,
    timestamp_yesterday: dt.datetime,
    helpers: StoreHelpers,
    caplog,
    mocker,
):
    """Test command with dry run options."""

    # GIVEN a case on disk that could be deleted
    caplog.set_level(logging.INFO)
    base_store = clean_context_microsalt.status_db
    helpers.add_analysis(
        base_store,
        pipeline=Pipeline.MICROSALT,
        started_at=timestamp_yesterday,
        uploaded_at=timestamp_yesterday,
        cleaned_at=None,
    )

    analysis_to_clean = base_store.family(microsalt_case_clean_dry).analyses[0]
    case_path_list = clean_context_microsalt.meta_apis["analysis_api"].get_case_path(
        microsalt_case_clean_dry
    )

    for path in case_path_list:
        Path(path).mkdir(exist_ok=True, parents=True)

    mocker.patch.object(TrailblazerAPI, "is_latest_analysis_ongoing")
    TrailblazerAPI.is_latest_analysis_ongoing.return_value = False

    # WHEN dry running with dry run specified
    result = cli_runner.invoke(
        clean_run_dir, [microsalt_case_clean_dry, "-d", "-y"], obj=clean_context_microsalt
    )

    # THEN command should say it would have deleted
    assert result.exit_code == EXIT_SUCCESS
    assert "Would have deleted" in caplog.text
    assert microsalt_case_clean_dry in caplog.text

    # THEN the analysis should still be in the analyses_to_clean query since this is a dry-ryn
    assert analysis_to_clean.cleaned_at == None
    assert analysis_to_clean in base_store.analyses_to_clean(pipeline=Pipeline.MICROSALT)


def test_clean_run(
    microsalt_case_clean: str,
    cli_runner: CliRunner,
    clean_context_microsalt: CGConfig,
    timestamp_yesterday: dt.datetime,
    helpers: StoreHelpers,
    caplog,
    mocker,
):
    """Test command with dry run options."""

    # GIVEN a case on disk that could be deleted
    caplog.set_level(logging.INFO)
    base_store = clean_context_microsalt.status_db
    helpers.add_analysis(
        base_store,
        pipeline=Pipeline.MICROSALT,
        started_at=timestamp_yesterday,
        uploaded_at=timestamp_yesterday,
        cleaned_at=None,
    )

    analysis_to_clean = base_store.family(microsalt_case_clean).analyses[0]
    case_path_list = clean_context_microsalt.meta_apis["analysis_api"].get_case_path(
        microsalt_case_clean
    )

    for path in case_path_list:
        Path(path).mkdir(exist_ok=True, parents=True)

    mocker.patch.object(TrailblazerAPI, "is_latest_analysis_ongoing")
    TrailblazerAPI.is_latest_analysis_ongoing.return_value = False

    # WHEN dry running with dry run specified
    result = cli_runner.invoke(
        clean_run_dir, [microsalt_case_clean, "-y"], obj=clean_context_microsalt
    )

    # THEN command should say it would have deleted
    assert result.exit_code == EXIT_SUCCESS
    assert microsalt_case_clean in caplog.text
    assert "Cleaned" in caplog.text

    # THEN the analysis should no longer be in the analyses_to_clean query
    assert isinstance(analysis_to_clean.cleaned_at, dt.datetime)
    assert analysis_to_clean not in base_store.analyses_to_clean(pipeline=Pipeline.MICROSALT)


def test_verify_case_paths_age(
    clean_context_microsalt: CGConfig, microsalt_case_clean: str, tmpdir_factory
):
    # GIVEN a list of run_dir paths for a microsalt case
    run_dir = [tmpdir_factory.mktemp("microsalt_case")]

    # WHEN at least one of the paths is younger than 21 days
    with pytest.raises(FileNotFoundError):
        # THEN a FileNotFoundError should be raised
        clean_context_microsalt.meta_apis["analysis_api"].verify_case_paths_age(
            run_dir, microsalt_case_clean
        )



"""Constants related to all things sequencing"""
from cgmodels.cg.constants import StrEnum


class Sequencers(StrEnum):
    """Sequencer instruments."""

    HISEQX: str = "hiseqx"
    HISEQGA: str = "hiseqga"
    NOVASEQ: str = "novaseq"
    ALL: str = "all"


sequencer_types = {
    "D00134": Sequencers.HISEQGA,
    "D00410": Sequencers.HISEQGA,
    "D00415": Sequencers.HISEQGA,
    "D00450": Sequencers.HISEQGA,
    "D00456": Sequencers.HISEQGA,
    "D00483": Sequencers.HISEQGA,
    "M03284": Sequencers.HISEQGA,
    "SN1025": Sequencers.HISEQGA,
    "SN7001298": Sequencers.HISEQGA,
    "SN7001301": Sequencers.HISEQGA,
    "ST-E00198": Sequencers.HISEQX,
    "ST-E00201": Sequencers.HISEQX,
    "ST-E00214": Sequencers.HISEQX,
    "ST-E00266": Sequencers.HISEQX,
    "ST-E00269": Sequencers.HISEQX,
    "A00187": Sequencers.NOVASEQ,
    "A00621": Sequencers.NOVASEQ,
    "A00689": Sequencers.NOVASEQ,
}


class SequencingMethod(StrEnum):
    """Sequencing method types."""

    TGS: str = "tgs"
    WES: str = "wes"
    WGS: str = "wgs"
    WTS: str = "wts"


class Variants(StrEnum):
    """Type of variants."""

    SNV: str = "snv"
    SV: str = "sv"
